@@ -15,6 +15,7 @@ from asyncio import (
     set_event_loop,
     sleep,
 )
+from contextlib import contextmanager
 from subprocess import Popen
 from traceback import format_tb
 from typing import (
@@ -23,7 +24,9 @@ from typing import (
     Callable,
     Dict,
     FrozenSet,
+    Generator,
     Generic,
+    Hashable,
     Iterable,
     List,
     Optional,
@@ -102,6 +105,9 @@ E = KeyPressEvent
 _AppResult = TypeVar("_AppResult")
 ApplicationEventHandler = Callable[["Application[_AppResult]"], None]
 
+_SIGWINCH = getattr(signal, "SIGWINCH", None)
+_SIGTSTP = getattr(signal, "SIGTSTP", None)
+
 
 class Application(Generic[_AppResult]):
     """
@@ -155,7 +161,7 @@ class Application(Generic[_AppResult]):
         don't want this for the implementation of a REPL. By default, this is
         enabled if `full_screen` is set.
 
-    Callbacks (all of these should accept a
+    Callbacks (all of these should accept an
     :class:`~prompt_toolkit.application.Application` object as input.)
 
     :param on_reset: Called during reset.
@@ -360,15 +366,23 @@ class Application(Generic[_AppResult]):
     @property
     def color_depth(self) -> ColorDepth:
         """
-        Active :class:`.ColorDepth`.
+        The active :class:`.ColorDepth`.
+
+        The current value is determined as follows:
+        - If a color depth was given explicitely to this application, use that
+          value.
+        - Otherwise, fall back to the color depth that is reported by the
+          :class:`.Output` implementation. If the :class:`.Output` class was
+          created using `output.defaults.create_output`, then this value is
+          coming from the $PROMPT_TOOLKIT_COLOR_DEPTH environment variable.
         """
         depth = self._color_depth
 
         if callable(depth):
-            return depth() or ColorDepth.default()
+            depth = depth()
 
         if depth is None:
-            return ColorDepth.default()
+            depth = self.output.get_default_color_depth()
 
         return depth
 
@@ -408,7 +422,7 @@ class Application(Generic[_AppResult]):
 
         self.exit_style = ""
 
-        self.background_tasks: List[Task] = []
+        self.background_tasks: List[Task[None]] = []
 
         self.renderer.reset()
         self.key_processor.reset()
@@ -526,14 +540,14 @@ class Application(Generic[_AppResult]):
         Start a while/true loop in the background for automatic invalidation of
         the UI.
         """
+        if self.refresh_interval not in (None, 0):
 
-        async def auto_refresh():
-            while True:
-                await sleep(self.refresh_interval)
-                self.invalidate()
+            async def auto_refresh(refresh_interval) -> None:
+                while True:
+                    await sleep(refresh_interval)
+                    self.invalidate()
 
-        if self.refresh_interval:
-            self.create_background_task(auto_refresh())
+            self.create_background_task(auto_refresh(self.refresh_interval))
 
     def _update_invalidate_events(self) -> None:
         """
@@ -624,7 +638,7 @@ class Application(Generic[_AppResult]):
             # pressed, we start a 'flush' timer for flushing our escape key. But
             # when any subsequent input is received, a new timer is started and
             # the current timer will be ignored.
-            flush_task: Optional[asyncio.Task] = None
+            flush_task: Optional[asyncio.Task[None]] = None
 
             # Reset.
             self.reset()
@@ -677,64 +691,55 @@ class Application(Generic[_AppResult]):
                     if self.input.closed:
                         f.set_exception(EOFError)
 
-            # Enter raw mode.
-            with self.input.raw_mode():
-                with self.input.attach(read_from_input):
-                    # Draw UI.
-                    self._request_absolute_cursor_position()
-                    self._redraw()
-                    self._start_auto_refresh_task()
+            # Enter raw mode, attach input and attach WINCH event handler.
+            with self.input.raw_mode(), self.input.attach(
+                read_from_input
+            ), attach_winch_signal_handler(self._on_resize):
+                # Draw UI.
+                self._request_absolute_cursor_position()
+                self._redraw()
+                self._start_auto_refresh_task()
 
-                    has_sigwinch = hasattr(signal, "SIGWINCH") and in_main_thread()
-                    if has_sigwinch:
-                        previous_winch_handler = signal.getsignal(signal.SIGWINCH)
-                        loop.add_signal_handler(signal.SIGWINCH, self._on_resize)
-
-                    # Wait for UI to finish.
+                # Wait for UI to finish.
+                try:
+                    result = await f
+                finally:
+                    # In any case, when the application finishes.
+                    # (Successful, or because of an error.)
                     try:
-                        result = await f
+                        self._redraw(render_as_done=True)
                     finally:
-                        # In any case, when the application finishes. (Successful,
-                        # or because of an error.)
-                        try:
-                            self._redraw(render_as_done=True)
-                        finally:
-                            # _redraw has a good chance to fail if it calls widgets
-                            # with bad code. Make sure to reset the renderer anyway.
-                            self.renderer.reset()
+                        # _redraw has a good chance to fail if it calls widgets
+                        # with bad code. Make sure to reset the renderer
+                        # anyway.
+                        self.renderer.reset()
 
-                            # Unset `is_running`, this ensures that possibly
-                            # scheduled draws won't paint during the following
-                            # yield.
-                            self._is_running = False
+                        # Unset `is_running`, this ensures that possibly
+                        # scheduled draws won't paint during the following
+                        # yield.
+                        self._is_running = False
 
-                            # Detach event handlers for invalidate events.
-                            # (Important when a UIControl is embedded in
-                            # multiple applications, like ptterm in pymux. An
-                            # invalidate should not trigger a repaint in
-                            # terminated applications.)
-                            for ev in self._invalidate_events:
-                                ev -= self._invalidate_handler
-                            self._invalidate_events = []
+                        # Detach event handlers for invalidate events.
+                        # (Important when a UIControl is embedded in multiple
+                        # applications, like ptterm in pymux. An invalidate
+                        # should not trigger a repaint in terminated
+                        # applications.)
+                        for ev in self._invalidate_events:
+                            ev -= self._invalidate_handler
+                        self._invalidate_events = []
 
-                            # Wait for CPR responses.
-                            if self.input.responds_to_cpr:
-                                await self.renderer.wait_for_cpr_responses()
+                        # Wait for CPR responses.
+                        if self.input.responds_to_cpr:
+                            await self.renderer.wait_for_cpr_responses()
 
-                            if has_sigwinch:
-                                loop.remove_signal_handler(signal.SIGWINCH)
-                                signal.signal(signal.SIGWINCH, previous_winch_handler)
+                        # Wait for the run-in-terminals to terminate.
+                        previous_run_in_terminal_f = self._running_in_terminal_f
 
-                            # Wait for the run-in-terminals to terminate.
-                            previous_run_in_terminal_f = self._running_in_terminal_f
+                        if previous_run_in_terminal_f:
+                            await previous_run_in_terminal_f
 
-                            if previous_run_in_terminal_f:
-                                await previous_run_in_terminal_f
-
-                            # Store unprocessed input as typeahead for next time.
-                            store_typeahead(
-                                self.input, self.key_processor.empty_queue()
-                            )
+                        # Store unprocessed input as typeahead for next time.
+                        store_typeahead(self.input, self.key_processor.empty_queue())
 
                 return result
 
@@ -807,9 +812,13 @@ class Application(Generic[_AppResult]):
             loop = new_event_loop()
             set_event_loop(loop)
 
-        return loop.run_until_complete(self.run_async(pre_run=pre_run))
+        return loop.run_until_complete(
+            self.run_async(pre_run=pre_run, set_exception_handler=set_exception_handler)
+        )
 
-    def _handle_exception(self, loop, context: Dict[str, Any]) -> None:
+    def _handle_exception(
+        self, loop: AbstractEventLoop, context: Dict[str, Any]
+    ) -> None:
         """
         Handler for event loop exceptions.
         This will print the exception, using run_in_terminal.
@@ -937,7 +946,7 @@ class Application(Generic[_AppResult]):
         self,
         command: str,
         wait_for_enter: bool = True,
-        display_before_text: str = "",
+        display_before_text: AnyFormattedText = "",
         wait_text: str = "Press ENTER to continue...",
     ) -> None:
         """
@@ -985,18 +994,18 @@ class Application(Generic[_AppResult]):
         """
         # Only suspend when the operating system supports it.
         # (Not on Windows.)
-        if hasattr(signal, "SIGTSTP"):
+        if _SIGTSTP is not None:
 
             def run() -> None:
-                # Send `SIGSTP` to own process.
+                # Send `SIGTSTP` to own process.
                 # This will cause it to suspend.
 
                 # Usually we want the whole process group to be suspended. This
                 # handles the case when input is piped from another process.
                 if suspend_group:
-                    os.kill(0, signal.SIGTSTP)
+                    os.kill(0, _SIGTSTP)
                 else:
-                    os.kill(os.getpid(), signal.SIGTSTP)
+                    os.kill(os.getpid(), _SIGTSTP)
 
             run_in_terminal(run)
 
@@ -1062,7 +1071,7 @@ class _CombinedRegistry(KeyBindingsBase):
         ] = SimpleCache()
 
     @property
-    def _version(self):
+    def _version(self) -> Hashable:
         """ Not needed - this object is not going to be wrapped in another
         KeyBindings object. """
         raise NotImplementedError
@@ -1155,11 +1164,11 @@ async def _do_wait_for_enter(wait_text: AnyFormattedText) -> None:
     key_bindings = KeyBindings()
 
     @key_bindings.add("enter")
-    def _(event: E) -> None:
+    def _ok(event: E) -> None:
         event.app.exit()
 
     @key_bindings.add(Keys.Any)
-    def _(event: E) -> None:
+    def _ignore(event: E) -> None:
         " Disallow typing. "
         pass
 
@@ -1167,3 +1176,48 @@ async def _do_wait_for_enter(wait_text: AnyFormattedText) -> None:
         message=wait_text, key_bindings=key_bindings
     )
     await session.app.run_async()
+
+
+@contextmanager
+def attach_winch_signal_handler(
+    handler: Callable[[], None]
+) -> Generator[None, None, None]:
+    """
+    Attach the given callback as a WINCH signal handler within the context
+    manager. Restore the original signal handler when done.
+
+    The `Application.run` method will register SIGWINCH, so that it will
+    properly repaint when the terminal window resizes. However, using
+    `run_in_terminal`, we can temporarily send an application to the
+    background, and run an other app in between, which will then overwrite the
+    SIGWINCH. This is why it's important to restore the handler when the app
+    terminates.
+    """
+    # The tricky part here is that signals are registered in the Unix event
+    # loop with a wakeup fd, but another application could have registered
+    # signals using signal.signal directly. For now, the implementation is
+    # hard-coded for the `asyncio.unix_events._UnixSelectorEventLoop`.
+
+    # No WINCH? Then don't do anything.
+    sigwinch = getattr(signal, "SIGWINCH", None)
+    if sigwinch is None or not in_main_thread():
+        yield
+        return
+
+    # Keep track of the previous handler.
+    # (Only UnixSelectorEventloop has `_signal_handlers`.)
+    loop = asyncio.get_event_loop()
+    previous_winch_handler = getattr(loop, "_signal_handlers", {}).get(sigwinch)
+
+    try:
+        loop.add_signal_handler(sigwinch, handler)
+        yield
+    finally:
+        # Restore the previous signal handler.
+        loop.remove_signal_handler(sigwinch)
+        if previous_winch_handler is not None:
+            loop.add_signal_handler(
+                sigwinch,
+                previous_winch_handler._callback,
+                *previous_winch_handler._args,
+            )
