@@ -18,6 +18,8 @@ from typing import (
     Union,
 )
 
+from prompt_toolkit.utils import is_dumb_terminal
+
 from ..key_binding import KeyPress
 from .base import Input
 from .posix_utils import PosixStdinReader
@@ -46,13 +48,13 @@ class Vt100Input(Input):
         try:
             # This should not raise, but can return 0.
             stdin.fileno()
-        except io.UnsupportedOperation:
+        except io.UnsupportedOperation as e:
             if "idlelib.run" in sys.modules:
                 raise io.UnsupportedOperation(
                     "Stdin is not a terminal. Running from Idle is not supported."
-                )
+                ) from e
             else:
-                raise io.UnsupportedOperation("Stdin is not a terminal.")
+                raise io.UnsupportedOperation("Stdin is not a terminal.") from e
 
         # Even when we have a file descriptor, it doesn't mean it's a TTY.
         # Normally, this requires a real TTY device, but people instantiate
@@ -65,6 +67,7 @@ class Vt100Input(Input):
         if not isatty and fd not in Vt100Input._fds_not_a_terminal:
             msg = "Warning: Input is not a terminal (fd=%r).\n"
             sys.stderr.write(msg % fd)
+            sys.stderr.flush()
             Vt100Input._fds_not_a_terminal.add(fd)
 
         #
@@ -75,7 +78,7 @@ class Vt100Input(Input):
         self._fileno = stdin.fileno()
 
         self._buffer: List[KeyPress] = []  # Buffer to collect the Key objects.
-        self.stdin_reader = PosixStdinReader(self._fileno)
+        self.stdin_reader = PosixStdinReader(self._fileno, encoding=stdin.encoding)
         self.vt100_parser = Vt100Parser(
             lambda key_press: self._buffer.append(key_press)
         )
@@ -86,6 +89,8 @@ class Vt100Input(Input):
         # It's not when the input is piped from Pexpect.
         if os.environ.get("PROMPT_TOOLKIT_NO_CPR", "") == "1":
             return False
+        if is_dumb_terminal():
+            return False
         try:
             return self.stdin.isatty()
         except ValueError:
