@@ -78,6 +78,10 @@ def load_basic_bindings() -> KeyBindings:
     @handle("f18")
     @handle("f19")
     @handle("f20")
+    @handle("f21")
+    @handle("f22")
+    @handle("f23")
+    @handle("f24")
     @handle("c-@")  # Also c-space.
     @handle("c-\\")
     @handle("c-]")
@@ -103,10 +107,10 @@ def load_basic_bindings() -> KeyBindings:
     @handle("pagedown")
     @handle("s-tab")
     @handle("tab")
-    @handle("s-c-left")
-    @handle("s-c-right")
-    @handle("s-c-home")
-    @handle("s-c-end")
+    @handle("c-s-left")
+    @handle("c-s-right")
+    @handle("c-s-home")
+    @handle("c-s-end")
     @handle("c-left")
     @handle("c-right")
     @handle("c-up")
@@ -117,7 +121,7 @@ def load_basic_bindings() -> KeyBindings:
     @handle("s-insert")
     @handle("c-insert")
     @handle(Keys.Ignore)
-    def _(event: E) -> None:
+    def _ignore(event: E) -> None:
         """
         First, for any of these keys, Don't do anything by default. Also don't
         catch them in the 'Any' handler which will insert them as data.
@@ -173,14 +177,14 @@ def load_basic_bindings() -> KeyBindings:
     )
 
     @handle("enter", filter=insert_mode & is_multiline)
-    def _(event: E) -> None:
+    def _newline(event: E) -> None:
         """
         Newline (in case of multiline input.
         """
         event.current_buffer.newline(copy_margin=not in_paste_mode())
 
     @handle("c-j")
-    def _(event: E) -> None:
+    def _newline2(event: E) -> None:
         r"""
         By default, handle \n as if it were a \r (enter).
         (It appears that some terminals send \n instead of \r when pressing
@@ -191,22 +195,22 @@ def load_basic_bindings() -> KeyBindings:
     # Delete the word before the cursor.
 
     @handle("up")
-    def _(event: E) -> None:
+    def _go_up(event: E) -> None:
         event.current_buffer.auto_up(count=event.arg)
 
     @handle("down")
-    def _(event: E) -> None:
+    def _go_down(event: E) -> None:
         event.current_buffer.auto_down(count=event.arg)
 
     @handle("delete", filter=has_selection)
-    def _(event: E) -> None:
+    def _cut(event: E) -> None:
         data = event.current_buffer.cut_selection()
         event.app.clipboard.set_data(data)
 
     # Global bindings.
 
     @handle("c-z")
-    def _(event: E) -> None:
+    def _insert_ctrl_z(event: E) -> None:
         """
         By default, control-Z should literally insert Ctrl-Z.
         (Ansi Ctrl-Z, code 26 in MSDOS means End-Of-File.
@@ -219,7 +223,7 @@ def load_basic_bindings() -> KeyBindings:
         event.current_buffer.insert_text(event.data)
 
     @handle(Keys.BracketedPaste)
-    def _(event: E) -> None:
+    def _paste(event: E) -> None:
         """
         Pasting from clipboard.
         """
@@ -238,7 +242,7 @@ def load_basic_bindings() -> KeyBindings:
         return get_app().quoted_insert
 
     @handle(Keys.Any, filter=in_quoted_insert, eager=True)
-    def _(event: E) -> None:
+    def _insert_text(event: E) -> None:
         """
         Handle quoted insert.
         """
