@@ -26,6 +26,7 @@ from typing import (
 from prompt_toolkit.data_structures import Size
 from prompt_toolkit.output import Output
 from prompt_toolkit.styles import ANSI_COLOR_NAMES, Attrs
+from prompt_toolkit.utils import is_dumb_terminal
 
 from .color_depth import ColorDepth
 
@@ -415,6 +416,7 @@ class Vt100_Output(Output):
         get_size: Callable[[], Size],
         term: Optional[str] = None,
         write_binary: bool = True,
+        default_color_depth: Optional[ColorDepth] = None,
     ) -> None:
 
         assert all(hasattr(stdout, a) for a in ("write", "flush"))
@@ -425,8 +427,9 @@ class Vt100_Output(Output):
         self._buffer: List[str] = []
         self.stdout = stdout
         self.write_binary = write_binary
+        self.default_color_depth = default_color_depth
         self._get_size = get_size
-        self.term = term or "xterm"
+        self.term = term
 
         # Cache for escape codes.
         self._escape_code_caches: Dict[ColorDepth, _EscapeCodeCache] = {
@@ -437,7 +440,12 @@ class Vt100_Output(Output):
         }
 
     @classmethod
-    def from_pty(cls, stdout: TextIO, term: Optional[str] = None) -> "Vt100_Output":
+    def from_pty(
+        cls,
+        stdout: TextIO,
+        term: Optional[str] = None,
+        default_color_depth: Optional[ColorDepth] = None,
+    ) -> "Vt100_Output":
         """
         Create an Output class from a pseudo terminal.
         (This will take the dimensions by reading the pseudo
@@ -446,12 +454,12 @@ class Vt100_Output(Output):
         # Normally, this requires a real TTY device, but people instantiate
         # this class often during unit tests as well. For convenience, we print
         # an error message, use standard dimensions, and go on.
-        isatty = stdout.isatty()
         fd = stdout.fileno()
 
-        if not isatty and fd not in cls._fds_not_a_terminal:
+        if not stdout.isatty() and fd not in cls._fds_not_a_terminal:
             msg = "Warning: Output is not a terminal (fd=%r).\n"
             sys.stderr.write(msg % fd)
+            sys.stderr.flush()
             cls._fds_not_a_terminal.add(fd)
 
         def get_size() -> Size:
@@ -460,11 +468,17 @@ class Vt100_Output(Output):
             # https://github.com/ipython/ipython/issues/10071
             rows, columns = (None, None)
 
-            if isatty:
+            # It is possible that `stdout` is no longer a TTY device at this
+            # point. In that case we get an `OSError` in the ioctl call in
+            # `get_size`. See:
+            # https://github.com/prompt-toolkit/python-prompt-toolkit/pull/1021
+            try:
                 rows, columns = _get_size(stdout.fileno())
+            except OSError:
+                pass
             return Size(rows=rows or 24, columns=columns or 80)
 
-        return cls(stdout, get_size, term=term)
+        return cls(stdout, get_size, term=term, default_color_depth=default_color_depth)
 
     def get_size(self) -> Size:
         return self._get_size()
@@ -677,3 +691,27 @@ class Vt100_Output(Output):
         " Sound bell. "
         self.write_raw("\a")
         self.flush()
+
+    def get_default_color_depth(self) -> ColorDepth:
+        """
+        Return the default color depth for a vt100 terminal, according to the
+        our term value.
+
+        We prefer 256 colors almost always, because this is what most terminals
+        support these days, and is a good default.
+        """
+        if self.default_color_depth is not None:
+            return self.default_color_depth
+
+        term = self.term
+
+        if term is None:
+            return ColorDepth.DEFAULT
+
+        if is_dumb_terminal(term):
+            return ColorDepth.DEPTH_1_BIT
+
+        if term in ("linux", "eterm-color"):
+            return ColorDepth.DEPTH_4_BIT
+
+        return ColorDepth.DEFAULT
