@@ -24,8 +24,7 @@ Example::
         s = PromptSession()
         result = s.prompt('Say something: ')
 """
-import os
-from asyncio import Future, get_event_loop
+from asyncio import get_event_loop
 from enum import Enum
 from functools import partial
 from typing import (
@@ -663,7 +662,7 @@ class PromptSession(Generic[_T]):
                         # The right prompt.
                         Float(
                             right=0,
-                            top=0,
+                            bottom=0,
                             hide_when_covering_content=True,
                             content=_RPrompt(lambda: self.rprompt),
                         ),
@@ -776,7 +775,7 @@ class PromptSession(Generic[_T]):
             )
 
         @handle("enter", filter=do_accept & default_focused)
-        def _(event: E) -> None:
+        def _accept_input(event: E) -> None:
             " Accept input when enter has been pressed. "
             self.default_buffer.validate_and_handle()
 
@@ -785,12 +784,12 @@ class PromptSession(Generic[_T]):
             return self.complete_style == CompleteStyle.READLINE_LIKE
 
         @handle("tab", filter=readline_complete_style & default_focused)
-        def _(event: E) -> None:
+        def _complete_like_readline(event: E) -> None:
             " Display completions (like Readline). "
             display_completions_like_readline(event)
 
         @handle("c-c", filter=default_focused)
-        def _(event: E) -> None:
+        def _keyboard_interrupt(event: E) -> None:
             " Abort when Control-C has been pressed. "
             event.app.exit(exception=KeyboardInterrupt, style="class:aborting")
 
@@ -805,7 +804,7 @@ class PromptSession(Generic[_T]):
             )
 
         @handle("c-d", filter=ctrl_d_condition & default_focused)
-        def _(event: E) -> None:
+        def _eof(event: E) -> None:
             " Exit when Control-D has been pressed. "
             event.app.exit(exception=EOFError, style="class:exiting")
 
@@ -816,7 +815,7 @@ class PromptSession(Generic[_T]):
             return to_filter(self.enable_suspend)()
 
         @handle("c-z", filter=suspend_supported & enable_suspend)
-        def _(event: E) -> None:
+        def _suspend(event: E) -> None:
             """
             Suspend process to background.
             """
@@ -870,6 +869,7 @@ class PromptSession(Generic[_T]):
         default: Union[str, Document] = "",
         accept_default: bool = False,
         pre_run: Optional[Callable[[], None]] = None,
+        set_exception_handler: bool = True,
     ) -> _T:
         """
         Display the prompt.
@@ -887,9 +887,6 @@ class PromptSession(Generic[_T]):
 
         Additional arguments, specific for this prompt:
 
-        :param _async: (internal, please call `prompt_async` instead). When
-            `True` return a `Future` instead of waiting for the prompt to
-            finish.
         :param default: The default input text to be shown. (This can be edited
             by the user).
         :param accept_default: When `True`, automatically accept the default
@@ -994,7 +991,7 @@ class PromptSession(Generic[_T]):
         if self._output is None and is_dumb_terminal():
             return get_event_loop().run_until_complete(self._dumb_prompt(self.message))
 
-        return self.app.run()
+        return self.app.run(set_exception_handler=set_exception_handler)
 
     async def _dumb_prompt(self, message: AnyFormattedText = "") -> _T:
         """
@@ -1088,6 +1085,7 @@ class PromptSession(Generic[_T]):
         default: Union[str, Document] = "",
         accept_default: bool = False,
         pre_run: Optional[Callable[[], None]] = None,
+        set_exception_handler: bool = True,
     ) -> _T:
 
         if message is not None:
@@ -1172,7 +1170,7 @@ class PromptSession(Generic[_T]):
         if self._output is None and is_dumb_terminal():
             return await self._dumb_prompt(self.message)
 
-        return await self.app.run_async()
+        return await self.app.run_async(set_exception_handler=set_exception_handler)
 
     def _add_pre_run_callables(
         self, pre_run: Optional[Callable[[], None]], accept_default: bool
